from .client import DuploClient
from .errors import DuploError

class DuploResource():
  
  def __init__(self, duplo: DuploClient):
    self.duplo = duplo
    self.tenant = None
  
  def get_tenant(self):
    if not self.tenant:
      self.tenant_svc = self.duplo.service("tenant")
      self.tenant = self.tenant_svc.find(self.duplo.tenant_name)
    return self.tenant
  
  def exec(self, subcmd, args=[]):
    if not (func := getattr(self, subcmd, None)):
      raise ValueError(f"Invalid subcommand: {subcmd}")
    try:
      res = func(*args)
      # if res is a dict or list, turn it into json
      if isinstance(res, (dict, list)):
        res = self.duplo.json(res)
      return print(res)
    except Exception as e:
      raise DuploError(f"Error executing subcommand: {subcmd}") from e
    
  



from duplocloud.client import DuploClient
from duplocloud.resource import DuploResource
from duplocloud.errors import DuploError

class DuploService(DuploResource):
  
  def __init__(self, duplo: DuploClient):
    super().__init__(duplo)
    self.tenent_svc = duplo.service('tenant')
  
  def list(self):
    """Retrieve a list of all services in a tenant."""
    tenant_id = self.get_tenant()["TenantId"]
    return self.duplo.get(f"subscriptions/{tenant_id}/GetReplicationControllers")
  
  def find(self, service_name):
    """Find a service by name.
    
    Args:
      service_name (str): The name of the service to find.
    Returns: 
      The service object.
    Raises:
      DuploError: If the service could not be found.
    """
    try:
      return [s for s in self.list() if s["Name"] == service_name][0]
    except IndexError:
      raise DuploError(f"Service '{service_name}' not found", 404)

  def update_image(self, service_name, image):
    """Update the image of a service.
    
    Args:
      service_name (str): The name of the service to update.
      image (str): The new image to use for the service.
    """
    tenant_id = self.get_tenant()["TenantId"]
    service = self.find(service_name)
    allocation_tags = service["Template"]["AllocationTags"]
    data = {
      "Name": service_name,
      "Image": image,
      "AllocationTags": allocation_tags
    }
    return self.duplo.post(f"subscriptions/{tenant_id}/ReplicationControllerChange", data)
  
  def restart(self, service_name):
    """Restart a service.
    
    Args:
      service_name (str): The name of the service to restart.
    Returns: 
      A success message if the service was restarted successfully.
    Raises:
      DuploError: If the service could not be restarted.
    """
    tenant_id = self.get_tenant()["TenantId"]
    res = self.duplo.post(f"subscriptions/{tenant_id}/ReplicationControllerReboot/{service_name}")
    if res.status_code == 200:
      return f"Successfully restarted service '{service_name}'"
    else:
      raise DuploError(f"Failed to restart service '{service_name}'")
  



import requests
import json
from cachetools import cached, TTLCache
from importlib.metadata import entry_points

ENTRYPOINT="duplocloud.net"

class DuploClient():
  """Duplo Client

  This is the main Duplo client class. It is used to connect to Duplo and
  to retrieve resources from Duplo. All services have a reference to this
  when they are created.

  Example: Using injected client to load a service.
      ```python
      from duplocloud.client import DuploClient  
      from duplocloud.resource import DuploResource  
      from duplocloud.errors import DuploError  

      class DuploSomeService(DuploResource):
        def __init__(self, duplo: DuploClient):
          super().__init__(duplo)
          self.tenent_svc = duplo.service('tenant')
      ```
  """
    
  def __init__(self, host, token, tenant_name="default", args=[]) -> None:
    self.host = host
    self.timeout = 10
    self.args = args
    self.tenant_name = tenant_name
    self.headers = {
      'Content-Type': 'application/json',
      'Authorization': f"Bearer {token}"
    }
  
  def __str__(self) -> str:
     return f"""
Client for Duplo at {self.host}
"""

  @cached(cache=TTLCache(maxsize=128, ttl=60))
  def get(self, path):
    """Get a Duplo resource.

    This request is cached for 60 seconds.
    
    Args:
      path: The path to the resource.
    Returns:
      The resource as a JSON object.
    """
    response = requests.get(
      url = f"{self.host}/{path}",
      headers = self.headers,
      timeout = self.timeout
    )
    return response.json()
  
  def post(self, path, data={}):
    """Post data to a Duplo resource.
    
    Args:
      path: The path to the resource.
      data: The data to post.
    Returns:
      The response as a JSON object.
    """
    return requests.post(
      url = f"{self.host}/{path}",
      headers = self.headers,
      timeout = self.timeout,
      json = data
    )
  
  def service(self, name):
    """Load Service
      
    Load a Service class from the entry points.

    Args:
      name: The name of the service.
    Returns:
      The instantiated service with a reference to this client.
    """
    eps = entry_points()[ENTRYPOINT]
    # e = entry_points(group=group, name=kind)
    e = [ep for ep in eps if ep.name == name][0]
    svc = e.load() 
    return svc(self)
  
  def json(self, data):
    """Convert data to JSON.
    
    Args:
      data: The data to convert.
    Returns:
      The data as a JSON object.
    """
    return json.dumps(data)


from .client import DuploClient
import argparse
import os

def main():
  env, args = load_env()
  client = DuploClient(
    host=env.host,
    token=env.token,
    tenant_name=env.tenant,
  )
  service = client.service(env.service)
  service.exec(env.subcmd, args)

def load_env():
  """Get the environment variables for the Duplo session."""
  parser = argparse.ArgumentParser(
    prog='duplocloud-cli',
    description='Duplo Cloud CLI',
  )
  parser.add_argument('service', help='The service to run')
  parser.add_argument('subcmd', help='The subcommand to run')
  parser.add_argument('-t', '--tenant', 
                      help='The tenant to be scope into',
                      default=os.getenv('DUPLO_TENANT', 'default'))
  parser.add_argument('-H', '--host', 
                      help='The tenant to be scope into',
                      default=os.getenv('DUPLO_HOST', None))
  parser.add_argument('-p', '--token', 
                      help='The token/password to authenticate with',
                      default=os.getenv('DUPLO_TOKEN', None))
  return parser.parse_known_args()


